@@ -1,13 +1,5 @@
 package Inline::CPP;
 
-#============================================================================
-# Editing Best practices:
-#     Unix line endings.
-#     Four spaces for indentation (no tabs).
-#     Conditional modifiers go on the following line, indented four spaces.
-#     Maintain Perl::Critic level 4 or better.
-#     Others as issues reveal themselves.
-#============================================================================
 
 use strict;
 use warnings;
@@ -17,6 +9,8 @@ require Inline::C;
 require Inline::CPP::grammar;
 require Inline::CPP::Config;
 
+# Note: Parse::RecDescent 'require'd within get_parser().
+
 use Carp;
 
 # use base doesn't work because Inline::C cannot be "use"d directly.
@@ -25,12 +19,11 @@ our @ISA = qw( Inline::C ); ## no critic (ISA)
 # Development releases will have a _0xx version suffix.
 # We eval the version number to accommodate dev. version numbering, as
 # described in perldoc perlmodstyle.
-our $VERSION = '0.38_004';
+our $VERSION = '0.40_001';
 $VERSION = eval $VERSION; ## no critic (eval)
 
 our $LOGFILE = q{c:/Users/daoswald/programming/repos/Inline-CPP/ilcpp.log};
 
-
 my $TYPEMAP_KIND;
 {
     no warnings qw/ once /; ## no critic (warnings)
@@ -55,20 +48,48 @@ sub register {
 # Validate the C++ config options: Now mostly done in Inline::C
 #============================================================================
 sub validate {
-    my ( $o, @config_options ) = @_;
-    # Set default compiler and libraries.
-    $o->{ILSM}{MAKEFILE}{CC}   ||= $Inline::CPP::Config::compiler;
-    $o->{ILSM}{MAKEFILE}{LIBS} ||= $Inline::CPP::Config::libs;
+    my ( $o, @config_options     )  =  @_;
+    my ( $flavor_defs, $iostream );
+
+    {   # "used only once" warning. We know it's ok.
+        no warnings qw( once ); ## no critic (warnings)
+        ## no critic (package variable)
+
+        # Set default compiler and libraries.
+        $o->{ILSM}{MAKEFILE}{CC}
+            ||= $Inline::CPP::Config::compiler;
+        $o->{ILSM}{MAKEFILE}{LIBS}
+            ||= _make_arrayref( $Inline::CPP::Config::libs );
+        
+        $flavor_defs = $Inline::CPP::Config::cpp_flavor_defs; # "Standard"?
+        $iostream    = $Inline::CPP::Config::iostream_fn; # iostream filename.
+    }
 
     # I haven't traced it out yet, but $o->{STRUCT} gets set before getting
     # properly set from Inline::C's validate().
     $o->{STRUCT} ||= {
-        '.macros' => q{},
-        '.xs'     => q{},
-        '.any'    => 0,
-        '.all'    => 0,
+        '.macros' => q{},   '.any' => 0,
+        '.xs'     => q{},   '.all' => 0,
     };
-    $o->{ILSM}{AUTO_INCLUDE} ||= <<'END';
+
+    _build_auto_include( $o, $flavor_defs, $iostream );
+
+    $o->{ILSM}{PRESERVE_ELLIPSIS} = 0
+        unless defined $o->{ILSM}{PRESERVE_ELLIPSIS};
+
+    # Filter out the parameters we treat differently than Inline::C,
+    # forwarding unknown requests back up to Inline::C.
+    my @propagate = _handle_config_options( $o, @config_options );
+    $o->SUPER::validate(@propagate) if @propagate;
+
+    return;
+}
+
+
+sub _build_auto_include {
+    my ( $o, $flavor_defs, $iostream ) = @_;
+    my $auto_include = <<'END';
+#define __INLINE_CPP 1
 #ifndef bool
 #include <%iostream%>
 #endif
@@ -83,45 +104,29 @@ extern "C" {
 #include <%iostream%>
 #endif
 
-
 END
 
-
-# Preprocessor definitions that will be defined if we're operating under
-# "Standard C++", and *not* if we're operating under pre-Standard.
-
-my $flavor_defs =  $Inline::CPP::Config::cpp_flavor_defs;
-
-    # Prepend the compiler flavor (Standard versus Legacy) #define's
-    # to the AUTO_INCLUDE boilerplate.  We prepend because that way
-    # it's easy for a user to #undef them in a custom-supplied
-    # AUTO_INCLUDE.  May be useful for overriding errant defaults,
-    # or testing.
-    $o->{ILSM}{AUTO_INCLUDE} =
-        $flavor_defs . $o->{ILSM}{AUTO_INCLUDE};
+    $o->{ILSM}{AUTO_INCLUDE} ||= $auto_include;
+    $o->{ILSM}{AUTO_INCLUDE} =   $flavor_defs  .  $o->{ILSM}{AUTO_INCLUDE};
+    # Replace %iostream% with the correct iostream library
+    $o->{ILSM}{AUTO_INCLUDE} =~ s{%iostream%}{$iostream}xg;
+    return;
+}
 
 
-    $o->{ILSM}{PRESERVE_ELLIPSIS} = 0
-      unless defined $o->{ILSM}{PRESERVE_ELLIPSIS};
-
-    # Filter out the parameters we treat differently than Inline::C
+sub _handle_config_options {
+    my ( $o, @config_options ) = @_;
     my @propagate;
+
     while ( @config_options ) {
         my ( $key, $value )
             = (  shift @config_options,  shift @config_options  );
         if ( $key eq 'LIBS' ) {
-            $value = [$value] unless ref $value eq 'ARRAY';
-            my $num = scalar @{ $o->{ILSM}{MAKEFILE}{LIBS} } - 1;
-            $o->{ILSM}{MAKEFILE}{LIBS}[$num] .= q{ } . $_
-                for ( @{$value} );
+            _handle_libs_cfg_option( $o, $value );
             next;
         }
         if ( $key eq 'ALTLIBS' ) {
-            $value = [$value] unless ref $value eq 'ARRAY';
-            push @{ $o->{ILSM}{MAKEFILE}{LIBS} }, q{};
-            my $num = scalar @{ $o->{ILSM}{MAKEFILE}{LIBS} } - 1;
-            $o->{ILSM}{MAKEFILE}{LIBS}[$num] .= q{ } . $_
-                for ( @{$value} );
+            _handle_altlibs_cfg_option( $o, $value );
             next;
         }
         if (     $key eq 'PRESERVE_ELLIPSIS'
@@ -135,17 +140,42 @@ my $flavor_defs =  $Inline::CPP::Config::cpp_flavor_defs;
         }
         push @propagate, $key, $value;
     }
+    return @propagate;
+}
 
-    # Replace %iostream% with the correct iostream library
-    my $iostream = $Inline::CPP::Config::iostream_fn;  # iostream filename
 
-    $o->{ILSM}{AUTO_INCLUDE} =~ s{%iostream%}{$iostream}xg;
-
-    # Forward all unknown requests up to Inline::C
-    $o->SUPER::validate(@propagate) if @propagate;
+sub _handle_libs_cfg_option {
+    my( $o, $value ) = @_;
+    $value = _make_arrayref( $value );
+    _add_libs( $o, $value );
     return;
 }
 
+
+sub _handle_altlibs_cfg_option {
+    my( $o, $value ) = @_;
+    $value = _make_arrayref( $value );
+    push @{ $o->{ILSM}{MAKEFILE}{LIBS} }, q{};
+    _add_libs( $o, $value );
+    return;
+}
+
+
+sub _make_arrayref {
+    my $value = shift;
+    $value = [ $value ] unless ref $value eq 'ARRAY';
+    return $value;
+}
+
+sub _add_libs {
+    my( $o, $libs ) = @_;
+    my $num = scalar @{ $o->{ILSM}{MAKEFILE}{LIBS} } - 1;
+    $o->{ILSM}{MAKEFILE}{LIBS}[$num] .= q{ } . $_
+        for @{ $libs };
+    return;
+}
+
+
 #============================================================================
 # Print a small report if PRINT_INFO option is set
 #============================================================================
@@ -271,24 +301,79 @@ sub xs_bindings {
     for my $class ( @{ $data->{classes} } ) {
         my $proper_pkg = $pkg . "::$class";
         # Set up the proper namespace
-        push @XS, <<"END";
+        push @XS, _build_namespace( $module, $proper_pkg );
+        push @XS,
+             _generate_member_xs_wrappers( $o, $pkg, $class, $proper_pkg );
+    }
+
+    push @XS, _remove_xs_prefixes ( $o, $module, $pkg );
+    push @XS, _generate_nonmember_xs_wrappers( $o );
+
+    for ( @{ $data->{enums} } ) {
+        # Global enums.
+        $o->{ILSM}{XS}{BOOT} .= make_enum(  $pkg, @{$_}{ qw( name body ) }  );
+    }
+    return join q{}, @XS;
+}
+
+
+# Set up the proper namespace.
+sub _build_namespace {
+    my ( $module, $proper_pkg ) = @_;
+    return <<"END";
 
 MODULE = $module        PACKAGE = $proper_pkg
 
 PROTOTYPES: DISABLE
 
 END
+}
 
-        my ( $ctor, $dtor, $abstract ) = ( 0, 0, 0 ); ## no critic (ambiguous)
-        for my $thing ( @{ $data->{class}{$class} } ) {
-            my ( $name, $scope, $type ) = @{$thing}{ qw| name scope thing | };
 
-            # Let Perl handle inheritance
-            if ( $type eq 'inherits' and $scope eq 'public' ) {
-                $o->{ILSM}{XS}{BOOT} ||= q{};
-                my $ISA_name = "${pkg}::${class}::ISA";
-                my $parent = "${pkg}::${name}";
-                $o->{ILSM}{XS}{BOOT} .= <<"END";
+sub _generate_member_xs_wrappers {
+    my( $o, $pkg, $class, $proper_pkg ) = @_;
+    my @XS;
+    my $data = $o->{ILSM}{parser}{data};
+    my ( $ctor, $dtor, $abstract ) = ( 0, 0, 0 ); ## no critic (ambiguous)
+    for my $thing ( @{ $data->{class}{$class} } ) {
+        my ( $name, $scope, $type ) = @{$thing}{ qw| name scope thing | };
+
+        _handle_inheritance ( $o, $type, $scope, $pkg, $class, $name );
+        # Get/set methods will go here:
+        # Cases we skip:
+        $abstract ||= ( $type eq 'method' and $thing->{abstract} );
+        next if ( $type eq 'method' and $thing->{abstract} );
+        next if   $scope ne 'public';
+        if ( $type eq 'enum' ) {
+            $o->{ILSM}{XS}{BOOT} .= make_enum(
+                $proper_pkg, $name, $thing->{body}
+            );
+        } elsif (  $type eq 'method'  and  $name !~ m/operator/  ) {
+            # generate an XS wrapper
+            $ctor ||= ( $name eq $class    );
+            $dtor ||= ( $name eq "~$class" );
+            push @XS, $o->wrap( $thing, $name, $class );
+        }
+    }
+
+    # Provide default constructor and destructor:
+    push @XS, "$class *\n${class}::new()\n\n"
+        unless ( $ctor or $abstract );
+
+    push @XS, "void\n${class}::DESTROY()\n\n"
+        unless ( $dtor or $abstract );
+    return @XS;
+}
+
+
+# Let Perl handle inheritance.
+sub _handle_inheritance {
+    my( $o, $type, $scope, $pkg, $class, $name ) = @_;
+    if ( $type eq 'inherits' and $scope eq 'public' ) {
+        $o->{ILSM}{XS}{BOOT} ||= q{};
+        my $ISA_name = "${pkg}::${class}::ISA";
+        my $parent = "${pkg}::${name}";
+        $o->{ILSM}{XS}{BOOT} .= <<"END";
 {
 #ifndef get_av
     AV *isa = perl_get_av("$ISA_name", 1);
@@ -297,72 +382,46 @@ END
 #endif
     av_push(isa, newSVpv("$parent", 0));
 }
-END
-            }
-
-            # Get/set methods will go here:
-
-            # Cases we skip:
-            $abstract ||= ( $type eq 'method' and $thing->{abstract} );
-            next if ( $type eq 'method' and $thing->{abstract} );
-            next if $scope ne 'public';
-            if ( $type eq 'enum' ) {
-                $o->{ILSM}{XS}{BOOT} .= make_enum(
-                    $proper_pkg, $name,
-                    $thing->{body}
-                );
-            } elsif ( $type eq 'method' ) {
-                next if $name =~ m/operator/;
-                # generate an XS wrapper
-                $ctor ||= ( $name eq $class    );
-                $dtor ||= ( $name eq "~$class" );
-                push @XS, $o->wrap( $thing, $name, $class );
-            }
-        }
-
-    # Provide default constructor and destructor:
-        push @XS, <<"END" unless ( $ctor or $abstract );
-$class *
-${class}::new()
-
-END
-        push @XS, <<"END" unless ( $dtor or $abstract );
-void
-${class}::DESTROY()
-
 END
     }
+    return;
+}
 
+
+sub _generate_nonmember_xs_wrappers {
+    my $o = shift;
+    my $data = $o->{ILSM}{parser}{data};
+    my @XS;
+    for my $function ( @{ $data->{functions} } ) {
+        # lose constructor defs outside class decls (and "implicit int")
+        next if $data->{function}{$function}{rtype} eq q{};
+        next if $data->{function}{$function}{rtype} =~ m/static/;  #specl case
+        next if $function =~ m/::/x;       # skip member functions.
+        next if $function =~ m/operator/;  # and operators.
+        push @XS, $o->wrap( $data->{function}{$function}, $function );
+    }
+    return @XS;
+}
+
+
+# Generate XS code to remove prefixes from function names.
+sub _remove_xs_prefixes {
+    my( $o, $module, $pkg ) = @_;
     my $prefix = (
         $o->{ILSM}{XS}{PREFIX}
             ? "PREFIX = $o->{ILSM}{XS}{PREFIX}"
             : q{}
     );
-    push @XS, <<"END";
+    return <<"END";
 MODULE = $module        PACKAGE = $pkg  $prefix
 
 PROTOTYPES: DISABLE
 
 END
 
-    for my $function ( @{ $data->{functions} } ) {
-        # lose constructor defs outside class decls (and "implicit int")
-        next if $data->{function}{$function}{rtype} eq q{};
-        next if $data->{function}{$function}{rtype} =~ m/static/;#specl case
-        next if $function =~ m/::/x;       # XXX: skip member functions?
-        next if $function =~ m/operator/;  # and operators.
-        push @XS, $o->wrap( $data->{function}{$function}, $function );
-    }
-
-    for ( @{ $data->{enums} } ) {
-        # Global enums.
-        $o->{ILSM}{XS}{BOOT} .= make_enum(  $pkg, @{$_}{ qw( name body ) }  );
-    }
-#     print "BOOT = \n", $o->{ILSM}{XS}{BOOT};
-
-    return join q{}, @XS;
 }
 
+
 #============================================================================
 # Generate an XS wrapper around anything: a C++ method or function
 #============================================================================
@@ -370,34 +429,26 @@ sub wrap {
     my( $o, $thing, $name, $class ) = @_;
     $class ||= q{};
     my $t  =   q{ } x 4; # indents in 4-space increments.
-
     my ( @XS, @PREINIT, @CODE );
-    my ( $ctor, $dtor ) = ( 0, 0 );
 
-    if ( $name eq $class ) {  # ctor
-        push @XS, $class . " *\n" . $class . '::new';
-        $ctor = 1;
-    }
-    elsif ( $name eq "~$class" ) { # dtor
-        push @XS, "void\n$class" . '::DESTROY';
-        $dtor = 1;
-    }
-    elsif ( $class ) {        # method
-        push @XS, "$thing->{rtype}\n$class" . "::$thing->{name}";
-    }
-    else {          # function
-        push @XS, "$thing->{rtype}\n$thing->{name}";
-    }
+    my ( $XS, $ctor, $dtor )
+        = _map_subnames_cpp_to_perl( $thing, $name, $class );
+
+    push @XS, $XS;
 
     return q{} unless $o->check_type( $thing, $ctor, $dtor );
 
     # Filter out optional subroutine arguments
     my ( @args, @opts, $ellipsis, $void );
+
     $_->{optional} ? push @opts, $_ : push @args, $_
         for @{ $thing->{args} };
+
     $ellipsis = pop @args
         if ( @args and $args[-1]{name} eq '...' );
+
     $void = ( $thing->{rtype} and $thing->{rtype} eq 'void' );
+
     push @XS, join q{}, (
         '(',
         join( ', ',
@@ -493,6 +544,29 @@ END
     return "@XS";
 }
 
+
+sub _map_subnames_cpp_to_perl {
+    my ( $thing, $name, $class ) = @_;
+    my ( $XS,    $ctor, $dtor  ) = ( q{}, 0, 0 );
+
+    if ( $name eq $class ) {  # ctor
+        $XS   = $class . " *\n" . $class . '::new';
+        $ctor = 1;
+    }
+    elsif ( $name eq "~$class" ) { # dtor
+        $XS   = "void\n$class" . '::DESTROY';
+        $dtor = 1;
+    }
+    elsif ( $class ) {        # method
+        $XS   = "$thing->{rtype}\n$class" . "::$thing->{name}";
+    }
+    else {          # function
+        $XS   = "$thing->{rtype}\n$thing->{name}";
+    }
+    return ( $XS, $ctor, $dtor );
+}
+
+
 sub call_or_instantiate {
     my (  $name,   $ctor,  $dtor,  $class, $const,  $type,  @args  ) = @_;
 
@@ -610,30 +684,14 @@ END
     foreach ( @{$body} ) {
         my ( $k, $v ) = @{$_};
         $val = $v if defined $v;
-        push @enum, <<"END";
-\tnewCONSTSUB(pkg, \"$k\", newSViv($val));
-END
+        push @enum, "\tnewCONSTSUB(pkg, \"$k\", newSViv($val));\n";
         ++$val;
     }
-    push @enum, <<"END";
-\t}
-END
+    push @enum, "\t}\n";
     return join q{}, @enum;
 }
 
 
-#    my $log = "name: $name, ctor: $ctor, dtor: $dtor, class: $class, const: $const, type, $type, args: (@args)\n";
-#    $log .= "\t" . const_cast( $rval, $const, $type ) . ";\n";
-
-# log_this( $LOGFILE, $log );
-#sub log_this {
-#    my ( $filename, @output ) = @_;
-#    open my $fh, '>>', $filename or die $!;
-#    print {$fh} @output, "\n";
-#    close $fh or die $!;
-#}
-
-
 1;
 
 __END__
