@@ -8,7 +8,7 @@ use vars qw($TYPEMAP_KIND $class_part $class_decl $star);
 # Dev versions will have a _0xx suffix.
 # We eval the $VERSION to accommodate dev version numbering as described in
 # perldoc perlmodstyle
-our $VERSION = '0.38_004';
+our $VERSION = '0.40_001';
 $VERSION = eval $VERSION;  ## no critic (eval)
 
 # Parse::RecDescent 1.90 and later have an incompatible change
@@ -509,3 +509,10 @@ sub strip_ellipsis {
 }
 
 1;
+
+__END__
+=head1 Inline::CPP::grammar
+
+All functions are internal.  No documentation necessary.
+
+=cut
