@@ -5,7 +5,7 @@ package Inline::CPP::Config;
 use strict;
 use warnings;
 
-our $VERSION = '0.38_004';
+our $VERSION = '0.40_001';
 $VERSION = eval $VERSION; ## no critic (eval)
 
 
@@ -33,3 +33,9 @@ END_FLAVOR_DEFINITIONS
 
 
 1;
+
+__END__
+
+=head1  Inline::CPP::Config
+
+For internal consumption; nothing to document.
